function [est] = ekf(sys, init, Z)
% Extended Kalman filter
%
%   Pedpokld se nsledujc model systmu (podrobnji viz dle):
%      x = phi(x) + w,
%      z = h(x) + v,
%   kde w ~ N(0, Q), v ~ N(0, R) a phi(x), h(x) jsou obecn nelinern 
%   funkce
%
%
%   [est] = ekf(sys, init, Z)
%
%   Vstupy: 
%   sys .... struktura reprezentujc model systmu
%            sys.phi ..... funkce stavov rovnice modelu phi(x)
%            sys.Phi ..... Jacobiho matice parcilnch derivac funkce phi 
%                          podle x
%            sys.h ....... funkce rovnice men modelu h(x)
%            sys.H ....... Jacobiho matice parcilnch derivac funkce h 
%                          podle x
%            sys.Q ....... kovariann matice systmovho umu w, n x n
%            sys.R ....... kovariann matice umu men v, l x l
%
%   init ... struktura reprezentujc poten podmnky
%            init.x ...... poten stav systmu, n x 1
%            init.P ...... kovariann matice potenho stavu, n x n
%
%   Z ...... matice, jej sloupce jsou vektory men z pro jednotliv
%            asov okamiky 1 : f, l x f
%   
%   Vstup: 
%   est .... pole struktur obsahujc vypotan odhady pro jednotliv 
%            asov okamiky 1 : f
%            est(k).x .... odhad stavu systmu v ase k
%            est(k).P .... chybov kovariann matice odhadu v ase k
%
%
%   Datum posledn pravy: 14. 1. 2018, KF


% INICIALIZAN ST

f = size(Z, 2);
n = size(init.x, 1);

x = init.x;
P = init.P;
est(1 : f) = struct('x', zeros(n, 1), 'P', zeros(n, n));

% VLASTN ALGORITMUS EKF

for k = 1 : f
    
    % Predikn krok (apriorn odhad)
    Phi = sys.Phi(x);    
    x = sys.phi(x);
    P = Phi * P * Phi' + sys.Q; 
    
    % Korekn krok (aposteriorn odhad)
    H = sys.H(x);
    K = P * H' * inv(H * P * H' + sys.R);
    x = x + K * (Z(:, k) - sys.h(x));
    P = P - K * H * P;    
        
    est(k).x = x;
    est(k).P = P;
    
end

end

