function [est] = kf(sys, init, Z, U)
% Kalmanv filtr
%
%   Pedpokld se nsledujc model systmu (podrobnji viz dle):
%      x = Phi * x + Gamma * u + G * w,
%      z = H * x + D * u + v,
%   kde w ~ N(0, Q), v ~ N(0, R)
%
%
%   [est] = kf(sys, init, Z, U)
%
%   Vstupy: 
%   sys .... struktura reprezentujc model systmu
%            sys.Phi ..... matice dynamiky systmu, n x n
%            sys.Gamma ... matice vlivu vstupu u na stav systmu x, n x r,
%                          (nepovinn, vchoz hodnota eye(n, r))                       
%            sys.G ....... matice vlivu umu w na stav systmu x, n x m,
%                          (nepovinn, vchoz hodnota eye(n, m))
%            sys.H ....... matice popisujc vztah mezi stavem systmu x 
%                          a menm z, l x n
%            sys.D ....... matice vlivu vstupu u na men z, l x r,
%                          (nepovinn, vchoz hodnota zeros(l, r))
%            sys.Q ....... kovariann matice systmovho umu w, m x m
%            sys.R ....... kovariann matice umu men v, l x l
%
%   init ... struktura reprezentujc poten podmnky
%            init.x ...... poten stav systmu, n x 1
%            init.P ...... kovariann matice potenho stavu, n x n
%
%   Z ...... matice, jej sloupce jsou vektory men z pro jednotliv
%            asov okamiky 1 : f, l x f
%
%   U ...... matice, jej sloupce jsou vektory vstup u pro jednotliv
%            asov okamiky 1 : f, r x f, (nepovinn, vchoz hodnota 
%            zeros(r, f))
%   
%   Vstup: 
%   est .... pole struktur obsahujc vypotan odhady pro jednotliv 
%            asov okamiky 1 : f
%            est(k).x .... odhad stavu systmu v ase k
%            est(k).P .... chybov kovariann matice odhadu v ase k
%
%
%   Datum posledn pravy: 18. 11. 2017, KF


% INICIALIZAN ST

[l, f] = size(Z);
n = size(init.x, 1);
m = size(sys.Q, 1);

% Nastaven vchozch hodnot nezadanch argument
if nargin == 3
   U = zeros(1, f);
end
r = size(U, 1);
if ~isfield(sys, 'Gamma') 
    sys.Gamma = eye(n, r); 
end
if ~isfield(sys, 'G') 
    sys.G = eye(n, m); 
end
if ~isfield(sys, 'D') 
    sys.D = zeros(l, r); 
end

x = init.x;
P = init.P;
est(1 : f) = struct('x', zeros(n, 1), 'P', zeros(n, n));

% VLASTN ALGORITMUS KALMANOVA FILTRU

for k = 1 : f
    
    % Predikn krok (apriorn odhad)
    x = sys.Phi * x + sys.Gamma * U(:, k);
    P = sys.Phi * P * sys.Phi' + sys.G * sys.Q * sys.G';
    
    % Korekn krok (aposteriorn odhad)
    K = P * sys.H' * inv(sys.H * P * sys.H' + sys.R);
    x = x + K * (Z(:, k) - sys.D * U(:, k) - sys.H * x);
    P = P - K * sys.H * P;
    
    est(k).x = x;
    est(k).P = P;
    
end

end

