function [est] = ukf(sys, init, Z, alpha, beta, kappa)
% Unscented Kalman filter
%
%   Pedpokld se nsledujc model systmu (podrobnji viz dle):
%      x = phi(x) + w,
%      z = h(x) + v,
%   kde w ~ N(0, Q), v ~ N(0, R) a phi(x), h(x) jsou obecn nelinern 
%   funkce
%
%
%   [est] = ukf(sys, init, Z, alpha, beta, kappa)
%
%   Vstupy: 
%   sys ..... struktura reprezentujc model systmu
%             sys.phi ..... funkce stavov rovnice modelu phi(x)
%             sys.h ....... funkce rovnice men modelu h(x)
%             sys.Q ....... kovariann matice systmovho umu w, n x n
%             sys.R ....... kovariann matice umu men v, l x l
%
%   init .... struktura reprezentujc poten podmnky
%             init.x ...... poten stav systmu, n x 1
%             init.P ...... kovariann matice potenho stavu, n x n
%
%   Z ....... matice, jej sloupce jsou vektory men z pro jednotliv
%             asov okamiky 1 : f, l x f
%
%   alpha, 
%   beta, 
%   kappa ... parametry pro unscented transform (nepovinn, vchoz
%             hodnoty: alpha = 1e-3, beta = 2, kappa = 0)
%   
%   Vstup: 
%   est ..... pole struktur obsahujc vypotan odhady pro jednotliv 
%             asov okamiky 1 : f
%             est(k).x .... odhad stavu systmu v ase k
%             est(k).P .... chybov kovariann matice odhadu v ase k
%
%
%   Datum posledn pravy: 3. 3. 2018, KF


% INICIALIZAN ST

f = size(Z, 2);
n = size(init.x, 1);
 
% Nastaven vchozch hodnot nezadanch argument
if nargin < 6
    kappa = 0;
end
if nargin < 5
    beta = 2;
end
if nargin < 4
    alpha = 1e-3;
end    

x = init.x;
P = init.P;
est(1 : f) = struct('x', zeros(n, 1), 'P', zeros(n, n));

% VLASTN ALGORITMUS UKF

for k = 1 : f

    % Predikn krok (apriorn odhad)
    [x, Pxx] = ut(x, P, sys.phi, alpha, beta, kappa);
    P = Pxx + sys.Q;
        
    % Korekn krok (aposteriorn odhad)
    [z, Pzz, Pxz] = ut(x, P, sys.h, alpha, beta, kappa);
    K = Pxz * inv(Pzz + sys.R);
    x = x + K * (Z(:, k) - z);
    P = P - K * Pxz';    
        
    est(k).x = x;
    est(k).P = P;
    
end

end

